@@ -1063,6 +1063,7 @@ def get_Ns(
     flag = True
     while flag:
         if age_is_max and (not comp_in_comp_end):
+            # method_use.old_neigh is ALWAYS False under this condition
             for i1 in tbl_index:
                 comp_M_local = exact_transferrables[i1][comp_M].values
                 df_local = pd.DataFrame({
@@ -1070,6 +1071,8 @@ def get_Ns(
                     'abs_comp_M_local': abs(comp_M_local)
                 })
                 agg_rem_local = df_local.min(axis = 1).values
+                if (agg_rem_local[:-2] == 0).all():
+                    agg_rem_local[-2:] = 0
                 agg_rem_local *= signs_for_receiver
                 comp_N_local = np.repeat([0], nrow)
                 local_rem = abs(agg_rem_local[-2:])
@@ -1092,6 +1095,8 @@ def get_Ns(
                 exact_transferrables[i1][comp_N] += comp_N_local
             flag = (agg_rem[-2:] != [0, 0]).any()
         else:
+            using_old_neigh =\
+                to_transfer_ages_comp[0] > to_transfer_ages_comp[-1]
             for i2 in tbl_index:
                 comp_M_local = exact_transferrables[i2][comp_M].values
                 df_local = pd.DataFrame({
@@ -1099,12 +1104,16 @@ def get_Ns(
                     'abs_comp_M_local': abs(comp_M_local)
                 })
                 agg_rem_local = df_local.min(axis = 1).values
+                if (agg_rem_local[:-1] == 0).all():
+                    agg_rem_local[-1] = 0
                 agg_rem_local *= signs_for_receiver
                 comp_N_local = np.repeat([0], nrow)
                 local_rem = abs(agg_rem_local[-1])
                 if local_rem >= 1:
                     comp_N_local[-1] = 1 if other_comp_in_neg else -1
-                    for j2 in range(nrow - 2, -1, -1):
+                    to_iterate = range(nrow - 1) if using_old_neigh \
+                        else range(nrow - 2, -1, -1)
+                    for j2 in to_iterate:
                         if abs(agg_rem_local[j2]) >= 1:
                             comp_N_local[j2] =\
                                 -1 if other_comp_in_neg else 1
