@@ -11,6 +11,7 @@ RegisteredOption = namedtuple(
 )
 age_down_to = 40
 age_floor = 50
+age_mask = []
 age_max = 99
 age_prop_size = 5
 comp_neg_temp_out = 'TEM'
@@ -33,6 +34,7 @@ comp_end = [comp_pos_npr_in]
 method_all = ['1dist', 'filler', 'prop']
 method = '1dist'
 method_use_age_floor = True
+method_use_old_neigh = False
 method_use_second_pass = True
 method_use_seq = True
 pop_age = 'Age'
@@ -78,14 +80,18 @@ def is_nonnegative(x, opname, force_positive = False):
         )
     return None
 
-def is_list_containing(x, opname, check_subset: bool = True):
+def is_list_of_item_type(x, opname, item_type):
     is_type(x, opname, list)
     for i, item in enumerate(x):
-        if not isinstance(item, str):
+        if not isinstance(item, item_type):
             raise ValueError(
                 f"{opname} has an item at index {i} that is not of "
-                f"type str but of {type(item)}."
+                f"type {item_type} but of {type(item)}."
             )
+    return None
+
+def is_list_containing(x, opname, check_subset: bool = True):
+    is_list_of_item_type(x, opname, str)
     if check_subset:
         comps =\
             [
@@ -115,6 +121,7 @@ def is_method(x):
 
 is_bool_afloor = lambda x: is_bool(x, 'method_use.age_floor')
 is_bool_mu2p = lambda x: is_bool(x, 'method_use.second_pass')
+is_bool_old_neigh = lambda x: is_bool(x, 'method_use.old_neigh')
 is_bool_seq = lambda x: is_bool(x, 'method_use.seq')
 is_containing_comp_end = lambda x: is_list_containing(x, 'comp.end', False)
 is_containing_netc = lambda x: is_list_containing(x, 'comp_neg.etc', False)
@@ -124,6 +131,7 @@ is_etc_p_b4_inters = lambda x: is_bool(x, 'comp_pos.put_etc_before')
 is_int_age_max = lambda x: is_int(x, 'age.max')
 is_int_age_down_to = lambda x: is_int(x, 'age.down_to')
 is_int_age_floor = lambda x: is_int(x, 'age.floor')
+is_list_str_age_mask = lambda x: is_list_of_item_type(x, 'age.mask', str)
 is_pos_age_prop_size = lambda x: is_nonnegative(x, 'age.prop_size', True)
 is_pos_pop_at_least = lambda x: is_nonnegative(x, 'pop.at_least')
 is_str_comp_neg_temp_out = lambda x: is_str(x, 'comp_neg.temp_out')
@@ -150,6 +158,7 @@ _global_config = {
     'age': {
         'down_to': age_down_to,
         'floor': age_floor,
+        'mask': age_mask,
         'max': age_max,
         'prop_size': age_prop_size
     },
@@ -179,6 +188,7 @@ _global_config = {
     'method': method,
     'method_use': {
         'age_floor': method_use_age_floor,
+        'old_neigh': method_use_old_neigh,
         'second_pass': method_use_second_pass,
         'seq': method_use_seq
     },
@@ -222,6 +232,18 @@ _registered_options = {
             "option will be ignored.",
         validator = is_int_age_floor
     ),
+    'age.mask': RegisteredOption(
+        key = 'age.mask',
+        defval = age_mask,
+        doc =\
+            "\n (list of str)\n    " +\
+            '(`[]` by default) A list of ages to "mask" or "ignore" ' +\
+            'in the end-of-period population table so that when these ' +\
+            'ages happen to be one of neighbouring ages of a certain ' +\
+            'problematic record, their end-of-period populations are ' +\
+            'considered to be 0.',
+        validator = is_list_str_age_mask
+    ),
     'age.max': RegisteredOption(
         key = 'age.max',
         defval = age_max,
@@ -438,6 +460,17 @@ _registered_options = {
             "neighbouring age of the problematic record.",
         validator = is_bool_afloor
     ),
+    'method_use.old_neigh': RegisteredOption(
+        key = 'method_use.old_neigh',
+        defval = method_use_old_neigh,
+        doc =\
+            "\n (bool)\n    " +\
+            '(False by default) If True, then the estime2 package ' +\
+            'considers ages older than the problematic age as ' +\
+            'neighbouring ages. If False, then the package considers ' +\
+            'ages younger than the problematic age as neighbouring ages.',
+        validator = is_bool_old_neigh
+    ),
     'method_use.second_pass': RegisteredOption(
         key = 'method_use.second_pass',
         defval = method_use_second_pass,
