@@ -1,6 +1,9 @@
 
 from estime2.age import Age
-from typing import List
+from typing import (
+    List, 
+    Optional
+)
 import numpy as np
 import pandas as pd
 
@@ -13,6 +16,7 @@ def compute_correction_evaluated_at_func(
     corrections_type: List,
     corrections_from: List,
     include_zeros: bool = False,
+    problematic_ages: List = [],
     *args, 
     **kwargs
 ):
@@ -21,7 +25,8 @@ def compute_correction_evaluated_at_func(
     evaluated at `func`. Provide more information about the corrections
     in `result_fix_issues` via `corrections_type` and `corrections_from`. 
     Name this quantity as `colname`. Specify positional arguments and 
-    keyword arguments of `func` if necessary.
+    keyword arguments of `func` if necessary. Provide `problematic_ages` of
+    each correction in `result_fix_issues`.
 
     Details
     -------
@@ -36,11 +41,12 @@ def compute_correction_evaluated_at_func(
     -----
     `compute_correction_evaluated_at_func(
         func, 
-        colname,
-        result_fix_issues,
-        corrections_type,
-        corrections_from,
-        include_zeros, 
+        colname: str,
+        result_fix_issues: List,
+        corrections_type: List,
+        corrections_from: List,
+        include_zeros: bool = False,
+        problematic_ages: List = [],
         *args, 
         **kwargs
     )`
@@ -62,6 +68,8 @@ def compute_correction_evaluated_at_func(
     * `include_zeros`: a bool, `False` by default; if `True`, then any
         corrections with the value 0 in the correction `Series` will be
         included in the quantity computation.
+    * `problematic_ages`: a list of `str`, `[]` by default; problematic 
+        ages of each corrections in `result_fix_issues`.
     * `*args`: positional arguments of `func`
     * `**kwargs`: keyword arguments of `func`
 
@@ -85,14 +93,16 @@ def compute_correction_evaluated_at_func(
         get_L_values = lambda sr: sr.iloc[:, -1]
     else:
         get_L_values = lambda sr: sr.iloc[:, -1].loc[lambda x: x != 0]
-    
+    p_ages_empty = problematic_ages == []
+
     dat = {}
     dat[pop_sex_age[0]] = []
-    dat[pop_sex_age[1]] = []
+    dat[pop_sex_age[1]] = [] if p_ages_empty else problematic_ages
     dat['Comp'] = []
     dat['Type'] = []
     dat['From'] = []
     dat[colname] = []
+    
     for i in range(1, len(result_fix_issues)):
         problematic_sex_age = result_fix_issues[i]\
             .iloc[-1, :]\
@@ -101,17 +111,19 @@ def compute_correction_evaluated_at_func(
         p_sex = problematic_sex_age[0]
         comp_L_values = get_L_values(result_fix_issues[i])
         p_comp = comp_L_values.name[:-2]
-        comp_in_comp_end = p_comp in result_fix_issues[0].get_comp_end()
-        modification_age = problematic_sex_age[1]
-        m_age_is_max = Age(modification_age).is_max()
-        p_age = None
-        if (not m_age_is_max) and (not comp_in_comp_end):
-            p_age = str(Age(modification_age) + 1)
-        else:
-            p_age = modification_age
+        if p_ages_empty == []:
+            comp_in_comp_end =\
+                p_comp in result_fix_issues[0].get_comp_end()
+            modification_age = problematic_sex_age[1]
+            m_age_is_max = Age(modification_age).is_max()
+            p_age = None
+            if (not m_age_is_max) and (not comp_in_comp_end):
+                p_age = str(Age(modification_age) + 1)
+            else:
+                p_age = modification_age
+            dat[pop_sex_age[1]].append(p_age)
         quant = func(comp_L_values, *args, **kwargs)
         dat[pop_sex_age[0]].append(p_sex)
-        dat[pop_sex_age[1]].append(p_age)
         dat['Comp'].append(p_comp)
         dat['Type'].append(corrections_type[i - 1])
         dat['From'].append(corrections_from[i - 1])
@@ -162,13 +174,17 @@ def get_agediff_range(
     correction.
     '''
 
+    result_fix_issues, p_ages =\
+        sanitize_result_fix_issues(result_fix_issues)
+
     result = compute_correction_evaluated_at_func(
         func = lambda sr: sr.index.max() - sr.index.min(),
         colname = 'range',
         result_fix_issues = result_fix_issues,
         corrections_type = corrections_type,
         corrections_from = corrections_from,
-        include_zeros = False
+        include_zeros = False,
+        problematic_ages = p_ages
     )
 
     return result
@@ -220,13 +236,17 @@ def get_agediff_sparsity(
     each correction.
     '''
 
+    result_fix_issues, p_ages =\
+        sanitize_result_fix_issues(result_fix_issues)
+
     result = compute_correction_evaluated_at_func(
-        func = lambda sr: np.mean(np.diff(sr.index)),
+        func = lambda sr: np.mean(abs(np.diff(sr.index))),
         colname = 'sparsity',
         result_fix_issues = result_fix_issues,
         corrections_type = corrections_type,
         corrections_from = corrections_from,
-        include_zeros = False
+        include_zeros = False,
+        problematic_ages = p_ages
     )
 
     return result
@@ -286,13 +306,17 @@ def get_correction_magni(
     computed at each correction.
     '''
 
+    result_fix_issues, p_ages =\
+        sanitize_result_fix_issues(result_fix_issues)
+
     result = compute_correction_evaluated_at_func(
         func = lambda sr: np.mean(np.abs(sr)),
         colname = 'magnitude',
         result_fix_issues = result_fix_issues,
         corrections_type = corrections_type,
         corrections_from = corrections_from,
-        include_zeros = include_zeros
+        include_zeros = include_zeros,
+        problematic_ages = p_ages
     )
 
     return result
@@ -351,6 +375,9 @@ def get_correction_sd(
     each correction.
     '''
 
+    result_fix_issues, p_ages =\
+        sanitize_result_fix_issues(result_fix_issues)
+
     result = compute_correction_evaluated_at_func(
         func = np.std,
         colname = 'sd',
@@ -358,6 +385,7 @@ def get_correction_sd(
         corrections_type = corrections_type,
         corrections_from = corrections_from,
         include_zeros = include_zeros,
+        problematic_ages = p_ages,
         ddof = 1
     )
 
@@ -437,3 +465,64 @@ def get_num_cells(
                 result.pop(k)
     
     return result
+
+def sanitize_result_fix_issues(result_fix_issues):
+    '''
+    If corrections are calculated under the strategy of using old 
+    neighbouring ages, then sort each correction of `result_fix_issues` by
+    `result_fix_issues[0].get_pop_groups()` and return problematic ages of
+    each correction.
+    '''
+
+    tbl_fixed = result_fix_issues[0]
+    corrections = result_fix_issues[1:]
+    pop_groups = tbl_fixed.get_pop_groups()
+    pop_age = pop_groups[1]
+
+    # See if each correction needs sorting
+    # It needs sorting if correction is applied under old_neigh
+    sample_correction = corrections[0].copy()
+    sample_correction_cp = sample_correction.copy()
+    sample_correction_cp[pop_age] =\
+        sample_correction_cp[pop_age].apply(Age)
+    sample_correction_cp = sample_correction_cp\
+        .sort_values(pop_groups)\
+        .reset_index(drop = True)
+    need_sorting =\
+        (
+            sample_correction[pop_age].apply(str).values !=\
+            sample_correction_cp[pop_age].apply(str).values
+        )\
+        .all()
+
+    # Get problematic ages of each correction
+    p_ages = []
+    for i in range(1, len(result_fix_issues)):
+        correction = result_fix_issues[i]
+        problematic_sex_age = correction\
+            .iloc[-1, :]\
+            [pop_groups]\
+            .values
+        p_comp = correction.columns[2][:-2]
+        comp_in_comp_end = p_comp in tbl_fixed.get_comp_end()
+        modification_age = problematic_sex_age[1]
+        m_age_is_max = Age(modification_age).is_max()
+        if (not m_age_is_max) and (not comp_in_comp_end):
+            p_ages.append(str(Age(modification_age) + 1))
+        else:
+            p_ages.append(modification_age)
+
+    # If corrections need sorting...
+    if need_sorting:
+        # Sort each correction by pop_groups
+        corrections_to_return = [df.copy() for df in corrections]
+        lsc = len(corrections_to_return)
+        for d in range(lsc):
+            corrections_to_return[d][pop_age] =\
+                corrections_to_return[d][pop_age].apply(Age)
+            corrections_to_return[d] = corrections_to_return[d]\
+                .sort_values(pop_groups)\
+                .reset_index(drop = True)
+        result_fix_issues = [tbl_fixed] + corrections_to_return
+
+    return result_fix_issues, p_ages
\ No newline at end of file
