@@ -5,6 +5,7 @@ from estime2.config import (
 )
 from estime2.poptable import (
     PopTable,
+    calculate_ages_to_modify_and_counter,
     return_goption_if_None
 )
 from estime2.poptable_resultswrapper import PopTableResultsWrapper
@@ -107,137 +108,11 @@ class ProvPopTable(PopTable):
         return_all_mods: bool = False
     ):
         '''
-        Apply modifications and counter-adjustments to `self` so that there
-        are no negative end-of-period population. Figures for counter-
-        adjustments in neighbouring ages are different based on `method`
-        specified. Set `return_all_mods` to `True` to get all the 
-        modifications and counter-adjustments made to `self`.
-
-        Usage
-        -----
-        `ProvPopTable.fix_issues(
-            method: Optional[str] = None, 
-            return_all_mods: bool = False
-        )`
-
-        Argument
-        --------
-        * `method`: the same argument as in `self.get_K(comp, method, J)`.
-        * `return_all_mods`: a bool (`False` by default); if `True`, this 
-            will return all the modifications and counter-adjustments made
-            to `self`.
-
-        Details
-        -------
-        For each problematic record, it modifies problematic record(s) and
-        makes counter-adjustments to records of neighbouring ages within a 
-        component according to the `method` specified. For each component, 
-        the modification and counter-adjustments made to that component of 
-        `self` will affect the end-of-period population. The process is 
-        done until no new end-of-period population is negative.
-        In case of `return_all_mods` is `True`, this method records how
-        much of modification and counter-adjustments are made, and returns
-        a `PopTableResultsWrapper` object.
-
-        Returns
-        -------
-        If `return_all_mods` is `False`, then it returns a fixed version
-        of `self`, the current `ProvPopTable`. 
-        If `return_all_mods` is `True`, then regardless of whether or not
-        there are corrections made to the original table, it returns a 
-        `PopTableResultsWrapper`.
+        Refer to the docstring of the `fix_issues(poptbl = self, 
+        method = method, return_all_mods = return_all_mods)` function.
         '''
 
-        args_in_self = self.get_args()
-        prevs = {}
-        for k, v in args_in_self.items():
-            prevs[k] = get_option(k)
-            set_option(k, v)
-
-        comp_neg_to_use = self.get_comp_neg()
-        comp_pos_to_use = self.get_comp_pos()
-        comps = comp_neg_to_use + comp_pos_to_use
-
-        pop_groups = self.get_pop_groups()
-        pop_sex = pop_groups[0]
-        all_cols = self.columns.tolist()
-        show_pop_end = self.get_pop_end() in all_cols
-        comps_modified = []
-        sexes = np.unique(self[pop_sex])
-        self_copy = None
-        method = return_goption_if_None('method', method)
-
-        for sex in sexes:
-            self_cp_sex =\
-                self.copy().loc[lambda df: df[pop_sex] == sex]
-            self_cp_sex = ProvPopTable(
-                self_cp_sex,
-                reorder_cols = False, # since it is reordered already
-                show_pop_end = show_pop_end,
-                # flag = False since it is grouped and summed already
-                flag = False
-            )
-            # `flag` is True iff `I` has at least one nonzero value
-            flag = (sum(self_cp_sex.get_I()['I'].values != 0) != 0)
-            i = 0
-            num_zeros = sum(self_cp_sex.get_I()['I'].values == 0)
-            while flag:
-                comp = comps[i]
-                self_cp_sex = apply_L(
-                    self_cp_sex, 
-                    comp, 
-                    method,
-                    comps_modified
-                )
-                I_vec = self_cp_sex.get_I()['I'].values
-                new_num_zeros = sum(I_vec == 0)
-                if all(I_vec == 0):
-                    # all problematic records fixed
-                    flag = False # stop the while-loop iteration
-                elif num_zeros != new_num_zeros:
-                    # one problematic record fixed
-                    i = 0 # move to the next problematic record
-                    num_zeros = new_num_zeros # assign a new number
-                elif i != (len(comps) - 1): 
-                    # correction in progress
-                    i += 1
-                else: 
-                    # i.e. lack of values for corrections
-                    # Correction could not be made to the problematic 
-                    # record at the youngest age.
-                    # If the youngest problematic record cannot be fixed, 
-                    # then any older problematic records cannot be fixed 
-                    # either.
-                    # intraprov_in or intraprov_out should be modified 
-                    # in SubProvPopTable
-                    flag = False
-
-            if self_copy is None:
-                self_copy = self_cp_sex
-            else:
-                self_copy = self_copy\
-                    .append(self_cp_sex, ignore_index = True)
-
-        self_copy = ProvPopTable(
-            self_copy.sort_values(pop_groups),
-            reorder_cols = False, # since it is reordered already
-            show_pop_end = show_pop_end,
-            # flag = False since it is grouped and summed already
-            flag = False
-        )
-
-        for k2, v2 in prevs.items():
-            set_option(k2, v2)
-
-        if return_all_mods:
-            result = PopTableResultsWrapper(
-                orig_table = self, 
-                result_fix_issues = [self_copy] + comps_modified,
-                method = method
-            )
-            return result
-        else:
-            return self_copy
+        return fix_issues(self, method, return_all_mods)
 
 class SubProvPopTable(PopTable):
     '''
@@ -424,137 +299,11 @@ class SubProvPopTable(PopTable):
         return_all_mods: bool = False
     ):
         '''
-        Apply modifications and counter-adjustments to `self` so that there
-        are no negative end-of-period population. Figures for counter-
-        adjustments in neighbouring ages are different based on `method`
-        specified. Set `return_all_mods` to `True` to get all the 
-        modifications and counter-adjustments made to `self`.
-
-        Usage
-        -----
-        `SubProvPopTable.fix_issues(
-            method: Optional[str] = None, 
-            return_all_mods: bool = False
-        )`
-
-        Argument
-        --------
-        * `method`: the same argument as in `self.get_K(comp, method, J)`.
-        * `return_all_mods`: a bool (`False` by default); if `True`, this 
-            will return all the modifications and counter-adjustments made
-            to `self`.
-
-        Details
-        -------
-        For each problematic record, it modifies problematic record(s) and
-        makes counter-adjustments to records of neighbouring ages within a 
-        component according to the `method` specified. For each component, 
-        the modification and counter-adjustments made to that component of 
-        `self` will affect the end-of-period population. The process is 
-        done until no new end-of-period population is negative.
-        In case of `return_all_mods` is `True`, this method records how
-        much of modification and counter-adjustments are made, and returns
-        a `PopTableResultsWrapper` object.
-
-        Returns
-        -------
-        If `return_all_mods` is `False`, then it returns a fixed version
-        of `self`, the current `SubProvPopTable`. 
-        If `return_all_mods` is `True`, then regardless of whether or not
-        there are corrections made to the original table, it returns a 
-        `PopTableResultsWrapper`.
+        Refer to the docstring of the `fix_issues(poptbl = self, 
+        method = method, return_all_mods = return_all_mods)` function.
         '''
 
-        args_in_self = self.get_args()
-        prevs = {}
-        for k, v in args_in_self.items():
-            prevs[k] = get_option(k)
-            set_option(k, v)
-
-        comp_neg_to_use = self.get_comp_neg()
-        comp_pos_to_use = self.get_comp_pos()
-        comps = comp_neg_to_use + comp_pos_to_use
-
-        pop_groups = self.get_pop_groups()
-        pop_sex = pop_groups[0]
-        all_cols = self.columns.tolist()
-        show_pop_end = self.get_pop_end() in all_cols
-        comps_modified = []
-        sexes = np.unique(self[pop_sex])
-        self_copy = None
-        method = return_goption_if_None('method', method)
-
-        for sex in sexes:
-            self_cp_sex =\
-                self.copy().loc[lambda df: df[pop_sex] == sex]
-            self_cp_sex = SubProvPopTable(
-                self_cp_sex,
-                reorder_cols = False, # since it is reordered already
-                show_pop_end = show_pop_end,
-                # flag = False since it is grouped and summed already
-                flag = False
-            )
-            # `flag` is True iff `I` has at least one nonzero value
-            flag = (sum(self_cp_sex.get_I()['I'].values != 0) != 0)
-            i = 0
-            num_zeros = sum(self_cp_sex.get_I()['I'].values == 0)
-            while flag:
-                comp = comps[i]
-                self_cp_sex = apply_L(
-                    self_cp_sex, 
-                    comp, 
-                    method,
-                    comps_modified
-                )
-                I_vec = self_cp_sex.get_I()['I'].values
-                new_num_zeros = sum(I_vec == 0)
-                if all(I_vec == 0):
-                    # all problematic records fixed
-                    flag = False # stop the while-loop iteration
-                elif num_zeros != new_num_zeros:
-                    # one problematic record fixed
-                    i = 0 # move to the next problematic record
-                    num_zeros = new_num_zeros # assign a new number
-                elif i != (len(comps) - 1): 
-                    # correction in progress
-                    i += 1
-                else: 
-                    # i.e. lack of values for corrections
-                    # Correction could not be made to the problematic 
-                    # record at the youngest age.
-                    # If the youngest problematic record cannot be fixed, 
-                    # then any older problematic records cannot be fixed 
-                    # either.
-                    # intraprov_in or intraprov_out should be modified 
-                    # in SubProvPopTable
-                    flag = False
-
-            if self_copy is None:
-                self_copy = self_cp_sex
-            else:
-                self_copy = self_copy\
-                    .append(self_cp_sex, ignore_index = True)
-
-        self_copy = SubProvPopTable(
-            self_copy.sort_values(pop_groups),
-            reorder_cols = False, # since it is reordered already
-            show_pop_end = show_pop_end,
-            # flag = False since it is grouped and summed already
-            flag = False
-        )
-
-        for k2, v2 in prevs.items():
-            set_option(k2, v2)
-
-        if return_all_mods:
-            result = PopTableResultsWrapper(
-                orig_table = self, 
-                result_fix_issues = [self_copy] + comps_modified,
-                method = method
-            )
-            return result
-        else:
-            return self_copy
+        return fix_issues(self, method, return_all_mods)
 
 def apply_L(
     poptbl: Union[ProvPopTable, SubProvPopTable], 
@@ -624,3 +373,251 @@ def apply_L(
         )
 
     return poptbl_cp
+
+def fix_issues(
+    poptbl: PopTable, 
+    method: Optional[str] = None, 
+    return_all_mods: bool = False
+):
+    '''
+    Apply modifications and counter-adjustments to `poptbl` so that there
+    are no negative end-of-period population. Figures for counter-
+    adjustments in neighbouring ages are different based on `method`
+    specified. Set `return_all_mods` to `True` to get all the 
+    modifications and counter-adjustments made to `poptbl`.
+
+    Usage
+    -----
+    `fix_issues(
+        poptbl,
+        method: Optional[str] = None,
+        return_all_mods: bool = False
+    )`
+
+    Argument
+    --------
+    * `poptbl`: the `PopTable` object, either `ProvPopTable` or 
+        `SubProvPopTable`
+    * `method`: the same argument as in `poptbl.get_K(comp, method, J)`.
+    * `return_all_mods`: a bool (`False` by default); if `True`, this 
+        will return all the modifications and counter-adjustments made
+        to `poptbl`.
+
+    Details
+    -------
+    For each problematic record, it modifies problematic record(s) and
+    makes counter-adjustments to records of neighbouring ages within a 
+    component according to the `method` specified. For each component, 
+    the modification and counter-adjustments made to that component of 
+    `poptbl` will affect the end-of-period population. The process is 
+    done until no new end-of-period population is negative.
+    In case of `return_all_mods` is `True`, this method records how
+    much of modification and counter-adjustments are made, and returns
+    a `PopTableResultsWrapper` object.
+
+    Returns
+    -------
+    If `return_all_mods` is `False`, then it returns a fixed version
+    of `poptbl`, the current `poptbl`. 
+    If `return_all_mods` is `True`, then regardless of whether or not
+    there are corrections made to the original table, it returns a 
+    `PopTableResultsWrapper`.
+    '''
+
+    poptbl_class = SubProvPopTable if poptbl.is_subprov() else ProvPopTable
+
+    args_in_self = poptbl.get_args()
+    prevs = {}
+    for k, v in args_in_self.items():
+        prevs[k] = get_option(k)
+        set_option(k, v)
+    
+    comp_neg_to_use = poptbl.get_comp_neg()
+    comp_pos_to_use = poptbl.get_comp_pos()
+    comps = comp_neg_to_use + comp_pos_to_use
+
+    pop_groups = poptbl.get_pop_groups()
+    pop_sex = pop_groups[0]
+    pop_age = pop_groups[1]
+    all_cols = poptbl.columns.tolist()
+    show_pop_end = poptbl.get_pop_end() in all_cols
+    sexes = np.unique(poptbl[pop_sex])
+    method = return_goption_if_None('method', method)
+
+    comps_modified = []
+    muon_pal = [(False, 1), (True, 1), (False, 0), (True, 0)]
+    
+    # set_option(...) statement here shows that the predefined values of
+    # `method_use.old_neigh` and `pop.at_least` will NOT be used.
+    # That is, whatever you defined outside of this function will NOT be
+    # used; only values in `muon_pal` will be used.
+    set_option(
+        'method_use.old_neigh', muon_pal[0][0],
+        'pop.at_least', muon_pal[0][1]
+    )
+    self_copy = produce_fixed_table(
+        poptbl,
+        sexes,
+        pop_sex,
+        pop_age,
+        poptbl_class,
+        show_pop_end,
+        comps,
+        method,
+        comps_modified
+    )
+    has_neg = (self_copy.calculate_pop().iloc[:, -1].values < 0).any()
+    mp = 1
+    while has_neg and mp != len(muon_pal):
+        set_option(
+            'method_use.old_neigh', muon_pal[mp][0],
+            'pop.at_least', muon_pal[mp][1]
+        )
+        self_copy = produce_fixed_table(
+            self_copy,
+            sexes,
+            pop_sex,
+            pop_age,
+            poptbl_class,
+            show_pop_end,
+            comps,
+            method,
+            comps_modified
+        )
+        has_neg = (self_copy.calculate_pop().iloc[:, -1].values < 0).any()
+        mp += 1
+
+    set_option(
+        'method_use.old_neigh', muon_pal[0][0],
+        'pop.at_least', muon_pal[0][1]
+    )
+    for k2, v2 in prevs.items():
+        set_option(k2, v2)
+
+    if return_all_mods:
+        if method == '1dist':
+            musp = get_option("method_use.second_pass")
+            museq = get_option("method_use.seq")
+            aps = get_option("age.prop_size")
+            pass2 = ", 2nd pass" if musp else ""
+            pass2seq =\
+                f", {aps}-year sequence" if (musp and museq) else ""
+            method = f"{method}{pass2}{pass2seq}"
+        result = PopTableResultsWrapper(
+            orig_table = poptbl, 
+            result_fix_issues = [self_copy] + comps_modified,
+            method = method
+        )
+        return result
+    else:
+        return self_copy
+
+def produce_fixed_table(
+    poptbl,
+    sexes,
+    pop_sex,
+    pop_age,
+    poptbl_class,
+    show_pop_end,
+    comps,
+    method,
+    comps_modified
+):
+    '''
+    Produce the fixed version of `poptbl` and return that fixed table.
+    Update `comps_modified` along the way.
+
+    Arguments
+    ---------
+    * `poptbl`: either the `ProvPopTable` or `SubProvPopTable` object
+    * `sexes`: a list-like; an array of all sexes in each table
+    * `pop_sex`: a str; the name of sex column
+    * `pop_age`: a str; the name of age column
+    * `poptbl_class`: either `ProvPopTable` or `SubProvPopTable`
+    * `show_pop_end`: a bool; Should the returning table display the
+            end-of-period populations?
+    * `comps`: a list; a list of all components in each table
+    * `method`: a str; one of "1dist", "filler", or "prop"
+    * `comps_modified`: a list that is going to store all the corrections
+        applied to `poptbl`
+    '''
+
+    self_copy = None
+    for sex in sexes:
+        self_cp_sex =\
+            poptbl.copy().loc[lambda df: df[pop_sex] == sex]
+        self_cp_sex = poptbl_class(
+            self_cp_sex,
+            reorder_cols = False, # since it is reordered already
+            show_pop_end = show_pop_end,
+            # flag = False since it is grouped and summed already
+            flag = False
+        )
+        # `flag` is True iff `I` has at least one nonzero value
+        flag = (sum(self_cp_sex.get_I()['I'].values != 0) != 0)
+        i = 0
+        num_zeros = sum(self_cp_sex.get_I()['I'].values == 0)
+        masked_ages = []
+        if flag:
+            prob_ages =\
+                self_cp_sex.get_I().query('I != 0')[pop_age].values
+            prob_ages = [str(a) for a in prob_ages.copy()]
+        while flag:
+            comp = comps[i]
+            current_prob_age = str(
+                self_cp_sex.get_I().query('I != 0')[pop_age].values[0]
+            )
+            self_cp_sex = apply_L(
+                self_cp_sex, 
+                comp, 
+                method,
+                comps_modified
+            )
+            I_vec = self_cp_sex.get_I()['I'].values
+            new_num_zeros = sum(I_vec == 0)
+            if all(I_vec == 0):
+                # all problematic records fixed
+                flag = False # stop the while-loop iteration
+            elif num_zeros != new_num_zeros:
+                # one problematic record fixed
+                i = 0 # move to the next problematic record
+                num_zeros = new_num_zeros # assign a new number
+            elif i != (len(comps) - 1): 
+                # correction in progress
+                i += 1
+            else: 
+                # i.e. lack of values for corrections
+                # Correction could not be made to the problematic 
+                # record at the current problematic age.
+                if current_prob_age == prob_ages[-1]:
+                    # The table `self_cp_sex` has no next problematic 
+                    # record (i.e. the current problematic record is 
+                    # the last problematic record in the table)
+                    flag = False
+                else:
+                    # The table does have the next problematic record.
+                    # Then, mask the current problematic record,
+                    # recompute neighbouring ages and try again.
+                    masked_ages.append(current_prob_age)
+                    set_option('age.mask', masked_ages)
+                    I_vec = self_cp_sex.get_I()['I'].values
+                    new_num_zeros = sum(I_vec == 0)
+                    i = 0
+                    num_zeros = new_num_zeros
+        set_option('age.mask', [])
+
+        if self_copy is None:
+            self_copy = self_cp_sex
+        else:
+            self_copy = self_copy\
+                .append(self_cp_sex, ignore_index = True)
+
+    self_copy = poptbl_class(
+        self_copy.sort_values([pop_sex, pop_age]),
+        reorder_cols = False, # since it is reordered already
+        show_pop_end = show_pop_end,
+        # flag = False since it is grouped and summed already
+        flag = False
+    )
+
+    return self_copy
